local addonName, addon = ...

LibStub('AceAddon-3.0'):NewAddon(addon, addonName, 'AceEvent-3.0')

local _G = _G

local trackPlayerBuff = {
	DEATHKNIGHT = {
		57330,		-- Horn of Winter
	},
	DRUID_103 = {
		1126,		-- Mark of the Wild
		52610,		-- Savage Roar
		127538,		-- Savage Roar with Glyph of Savagery
	},
	ROGUE = {
		[2823] = { 2823, 3408, 5761, 8679 },	-- Deadly Poison
		[3408] = { 2823, 3408, 5761, 8679 },	-- Crippling Poison
		[5761] = { 2823, 3408, 5761, 8679 },	-- Mind-numbing Poison
		[8679] = { 2823, 3408, 5761, 8679 },	-- Wound Poison
	},
}

local trackEnemyBuff = {
	DEATHKNIGHT = {
		[55078] = { 45462 },		-- Blood Plague = Plague Strike
		[55095] = { 45477, 49184 },	-- Frost Fever = Icy Touch or Howling Blast
	},
	DRUID_103 = {
		770,		-- Faerie Fire
		1079,		-- Rip
		1822,		-- Rake
	},
	ROGUE = {
	},
}

local trackPlayerCombo = {
	DRUID_103 = {
		count = 5,
		list = {
			1079,		-- Rip
			22568,		-- Ferocious Bite
			52610,		-- Savage Roar
			127538,		-- Savage Roar with Glyph of Savagery
		},
	},
	ROGUE = {
		count = 5,
		list = {
		},
	},
}

function addon:OnInitialize()
	self.spellState = {}

	self:RegisterEvent('PLAYER_ENTERING_WORLD', function(...)
		addon:OnPlayerEnteringWorld(...)
	end)

	self:RegisterEvent('PLAYER_TARGET_CHANGED', function(...)
		addon:OnPlayerTargetChanged(...)
	end)

	self:RegisterEvent('UNIT_AURA', function(...)
		addon:OnUnitAura(...)
	end)

	self:RegisterEvent('UNIT_COMBO_POINTS', function(...)
		addon:OnUnitComboPoints(...)
	end)
end

function addon:OnEnable()
end

function addon:OnDisable()
end

function addon:OnPlayerEnteringWorld(event)
	self:UpdatePlayerBuff()
	self:UpdateEnemyBuff()
	self:UpdatePlayerCombo()

	self:UpdateButtons()
end

function addon:OnPlayerTargetChanged(event)
	if UnitGUID('target') then
		if not UnitIsFriend('target', 'player') then
			self:UpdateEnemyBuff()
			self:UpdatePlayerCombo()
		end
	end

	self:UpdateButtons()
end

function addon:OnUnitAura(event, unit)
	if unit == 'player' then
		self:UpdatePlayerBuff()
		self:UpdateButtons()

	elseif unit == 'target' then
		self:UpdateEnemyBuff()
		self:UpdateButtons()
	end
end

function addon:OnUnitComboPoints(evene, unit)
	if unit == 'player' then
		self:UpdatePlayerCombo()
		self:UpdateButtons()
	end
end

function addon:UpdatePlayerBuff()
	local className = select(2, UnitClass('player'))

	local spec = GetSpecialization()
	local specId = spec and select(1, GetSpecializationInfo(spec)) or 0;

	local trackTable = trackPlayerBuff[className .. '_' .. specId] or trackPlayerBuff[className]

	if trackTable then
		local turnOff, turnOn = {}, {}

		local unitBuffs = {}
		local i = 1
		local buffId = select(11, UnitBuff('player', i, 'PLAYER'))

		while buffId do
			unitBuffs[buffId] = 1

			i = i + 1
			buffId = select(11, UnitBuff('player', i, 'PLAYER'))
		end

		self:UpdateBuffOnOffState(trackTable, unitBuffs, turnOn, turnOff)

		self:UpdateSpellState('player', turnOn, turnOff)
	end
end

function addon:UpdateEnemyBuff()
	local className = select(2, UnitClass('player'))

	local spec = GetSpecialization()
	local specId = spec and select(1, GetSpecializationInfo(spec)) or 0;

	local trackTable = trackEnemyBuff[className .. '_' .. specId] or trackEnemyBuff[className]

	if trackTable then
		local turnOff, turnOn = {}, {}

		local unitBuffs = {}
		local i = 1
		local buffId = select(11, UnitDebuff('target', i, 'PLAYER'))

		while buffId do
			unitBuffs[buffId] = 1

			i = i + 1
			buffId = select(11, UnitDebuff('target', i, 'PLAYER'))
		end

		self:UpdateBuffOnOffState(trackTable, unitBuffs, turnOn, turnOff)

		self:UpdateSpellState('enemy', turnOn, turnOff)
	end
end

function addon:UpdateBuffOnOffState(trackTable, unitBuffs, turnOn, turnOff)
	local buffId, spellList, spellId

	for buffId, spellList in pairs(trackTable) do
		if type(spellList) ~= 'table' then
			buffId = spellList
			spellList = { spellList }
		end

		if unitBuffs[buffId] then
			for _, spellId in pairs(spellList) do
				turnOff[spellId] = 1
			end
		else
			for _, spellId in pairs(spellList) do
				turnOn[spellId] = 1
			end
		end
	end
end

function addon:UpdatePlayerCombo()
	local className = select(2, UnitClass('player'))

	local spec = GetSpecialization()
	local specId = spec and select(1, GetSpecializationInfo(spec)) or 0;

	local trackTable = trackPlayerCombo[className .. '_' .. specId] or trackPlayerCombo[className]

	if trackTable then
		local turnOn, turnOff = {}, {}

		local spellId

		if GetComboPoints('player', 'target') >= trackTable.count then
			for _, spellId in pairs(trackTable.list) do
				turnOn[spellId] = 1
			end
		else
			for _, spellId in pairs(trackTable.list) do
				turnOff[spellId] = 1
			end
		end

		self:UpdateSpellState('combo', turnOn, turnOff)
	end
end

function addon:UpdateSpellState(layer, turnOn, turnOff)
	local spellId

	for spellId in pairs(turnOn) do
		if not self.spellState[spellId] then
			self.spellState[spellId] = {}
		end

		self.spellState[spellId][layer] = 1
	end

	for spellId in pairs(turnOff) do
		if not self.spellState[spellId] then
			self.spellState[spellId] = {}
		end

		self.spellState[spellId][layer] = nil
	end
end

function addon:UpdateButtons()
	local layers = { player = 1, combo = 1 }

	if UnitGUID('target') then
		if not UnitIsFriend('target', 'player') then
			layers.enemy = 1
		end
	end

	local i
	for i = 1, 12 do
		self:UpdateButton(_G["ActionButton" .. i], layers)
		self:UpdateButton(_G["MultiBarBottomLeftButton" .. i], layers)
		self:UpdateButton(_G["MultiBarBottomRightButton" .. i], layers)
		self:UpdateButton(_G["MultiBarRightButton" .. i], layers)
		self:UpdateButton(_G["MultiBarLeftButton" .. i], layers)
	end
end

function addon:UpdateButton(button, layers)
	local overlay = _G[button:GetName() .. 'SDLoverlay']

	if not overlay then
		local width, height = button:GetSize()

		overlay = CreateFrame('Frame', button:GetName() .. 'SDLoverlay', button)

		overlay:ClearAllPoints()
		overlay:SetSize(width, height)

		overlay:SetPoint('TOPLEFT', button, 'TOPLEFT', 0, 0)
		overlay:SetPoint('BOTTOMRIGHT', button, 'BOTTOMRIGHT', 0, 0)

		overlay.hl = overlay:CreateTexture()

		overlay.hl:SetAllPoints(overlay)
		overlay.hl:SetTexture([[Interface\Buttons\CheckButtonHilight]])
		overlay.hl:SetBlendMode('ADD')

		overlay:Hide()
	end

	local type, id = GetActionInfo(button.action)
	local state = 0

	if type == 'spell' and self.spellState[id] then
		local layer
		for layer in pairs(layers) do
			if self.spellState[id][layer] then
				state = state + 1
			end
		end
	end

	if state > 0 then
		button.SDLstate = state

		overlay:Show()
		overlay:SetAlpha(state * 0.6)
	else
		button.SDLstate = 0
		overlay:Hide()
	end
end
